@@ -11,7 +11,7 @@ URL_BASE = 'http://test.openvidu.io:4443/'
 SESSIONS = {"numberOfElements": 2, "content": [
     {"sessionId": "TestSession", "createdAt": 1538482606338, "mediaMode": "ROUTED", "recordingMode": "MANUAL",
      "defaultOutputMode": "COMPOSED", "defaultRecordingLayout": "BEST_FIT", "customSessionId": "TestSession",
-     "connections": {"numberOfElements": 2, "content": [
+     "connections": {"numberOfElements": 3, "content": [
          {"connectionId": "vhdxz7abbfirh2lh", "createdAt": 1538482606412, "location": "",
           "platform": "Chrome 69.0.3497.100 on Linux 64-bit",
           "token": "wss://localhost:4443?sessionId=TestSession&token=2ezkertrimk6nttk&role=PUBLISHER&turnUsername=H0EQLL&turnCredential=kjh48u",
@@ -20,16 +20,27 @@ SESSIONS = {"numberOfElements": 2, "content": [
               "mediaOptions": {"hasAudio": True, "audioActive": True, "hasVideo": True, "videoActive": True,
                                "typeOfVideo": "CAMERA", "frameRate": 30,
                                "videoDimensions": "{\"width\":640,\"height\":480}", "filter": {}}}],
-          "subscribers": []}, {"connectionId": "maxawd3ysuj1rxvq", "createdAt": 1538482607659, "location": "",
-                               "platform": "Chrome 69.0.3497.100 on Linux 64-bit",
-                               "token": "wss://localhost:4443?sessionId=TestSession&token=ovj1b4ysuqmcirti&role=PUBLISHER&turnUsername=INOAHN&turnCredential=oujrqd",
-                               "role": "PUBLISHER", "serverData": "", "clientData": "TestClient2", "publishers": [],
-                               "subscribers": [
-                                   {"createdAt": 1538482607799, "streamId": "vhdxz7abbfirh2lh_CAMERA_CLVAU"
-                                    }]}]}, "recording": False},
+          "subscribers": []},
+         {"connectionId": "maxawd3ysuj1rxvq", "createdAt": 1538482607659, "location": "",
+          "platform": "Chrome 69.0.3497.100 on Linux 64-bit",
+          "token": "wss://localhost:4443?sessionId=TestSession&token=ovj1b4ysuqmcirti&role=PUBLISHER&turnUsername=INOAHN&turnCredential=oujrqd",
+          "role": "PUBLISHER", "serverData": "", "clientData": "TestClient2", "publishers": [],
+          "subscribers": [
+              {"createdAt": 1538482607799, "streamId": "vhdxz7abbfirh2lh_CAMERA_CLVAU"
+               }
+          ]},
+         {"connectionId": "maxawc4zsuj1rxva", "createdAt": 1538482607659, "location": "",
+          "platform": "Chrome 69.0.3497.100 on Linux 64-bit",
+          "token": "wss://localhost:4443?sessionId=TestSession&token=ovj1b4ysuqmcirti&role=PUBLISHER&turnUsername=INOAHN&turnCredential=oujrqd",
+          "role": "PUBLISHER", "publishers": [],
+          "subscribers": [
+              {"createdAt": 1538482607799, "streamId": "vhdxz7abbfirh2lh_CAMERA_CLVAU"
+               }
+          ]},
+     ]}, "recording": False},
     {"sessionId": "TestSession2", "createdAt": 1538482606338, "mediaMode": "ROUTED", "recordingMode": "MANUAL",
      "defaultOutputMode": "COMPOSED", "defaultRecordingLayout": "BEST_FIT", "customSessionId": "TestSession",
-     "connections": {"numberOfElements": 2, "content": [
+     "connections": {"numberOfElements": 3, "content": [
          {"connectionId": "vhdxz7abbfirh2lh", "createdAt": 1538482606412, "location": "",
           "platform": "Chrome 69.0.3497.100 on Linux 64-bit",
           "token": "wss://localhost:4443?sessionId=TestSession&token=2ezkertrimk6nttk&role=PUBLISHER&turnUsername=H0EQLL&turnCredential=kjh48u",
@@ -43,7 +54,17 @@ SESSIONS = {"numberOfElements": 2, "content": [
                                "token": "wss://localhost:4443?sessionId=TestSession&token=ovj1b4ysuqmcirti&role=PUBLISHER&turnUsername=INOAHN&turnCredential=oujrqd",
                                "role": "PUBLISHER", "serverData": "", "clientData": "TestClient2", "publishers": [],
                                "subscribers": [
-                                   {"createdAt": 1538482607799, "streamId": "vhdxz7abbfirh2lh_CAMERA_CLVAU"}]}]},
+                                   {"createdAt": 1538482607799, "streamId": "vhdxz7abbfirh2lh_CAMERA_CLVAU"}]},
+         {"connectionId": "ipc_IPCAM_rtsp_A8MJ_91_191_213_49_554_live_mpeg4_sdp", "createdAt": 1582121476379,
+          "location": "unknown", "platform": "IPCAM", "role": "PUBLISHER", "serverData": "MY_IP_CAMERA", "publishers": [
+             {"createdAt": 1582121476439,
+              "streamId": "str_IPC_XC1W_ipc_IPCAM_rtsp_A8MJ_91_191_213_49_554_live_mpeg4_sdp",
+              "rtspUri": "rtsp://91.191.213.49:554/live_mpeg4.sdp",
+              "mediaOptions": {"hasAudio": True, "audioActive": True, "hasVideo": True, "videoActive": True,
+                               "typeOfVideo": "IPCAM", "frameRate": None, "videoDimensions": None, "filter": {},
+                               "adaptativeBitrate": True, "onlyPlayWithSubscribers": True}}], "subscribers": []}
+
+     ]},
      "recording": False}
 ]}
 
@@ -257,10 +278,10 @@ def test_inlist_after_close_without_fetch(openvidu_instance, requests_mock):
 def test_fetching_changed(openvidu_instance, requests_mock):
     session_before_change = openvidu_instance.get_session('TestSession')
 
-    assert session_before_change.connection_count == 2
+    assert session_before_change.connection_count == 3
 
     original = deepcopy(SESSIONS)  # Deep copy
-    original['content'][0]['connections']['numberOfElements'] = 3
+    original['content'][0]['connections']['numberOfElements'] = 4
     original['content'][0]['connections']['content'].append(
         {"connectionId": "vhdxz7a3bfirh2lh", "createdAt": 1538482606412, "location": "",
          "platform": "Chrome 69.0.3497.100 on Linux 64-bit",
@@ -276,7 +297,7 @@ def test_fetching_changed(openvidu_instance, requests_mock):
 
     is_changed = openvidu_instance.fetch()
 
-    assert session_before_change.connection_count == 3
+    assert session_before_change.connection_count == 4
     assert is_changed
 
 
