@@ -4,7 +4,7 @@
 
 import pytest
 from datetime import datetime
-from pyopenvidu import OpenVidu, OpenViduSessionDoesNotExistsError, OpenViduConnectionDoesNotExistsError
+from pyopenvidu import OpenVidu, OpenViduError, OpenViduSessionDoesNotExistsError, OpenViduConnectionDoesNotExistsError
 from urllib.parse import urljoin
 from copy import deepcopy
 
@@ -12,7 +12,7 @@ URL_BASE = 'http://test.openvidu.io:4443/'
 SESSIONS = {"numberOfElements": 2, "content": [
     {"sessionId": "TestSession", "createdAt": 1538482606338, "mediaMode": "ROUTED", "recordingMode": "MANUAL",
      "defaultOutputMode": "COMPOSED", "defaultRecordingLayout": "BEST_FIT", "customSessionId": "TestSession",
-     "connections": {"numberOfElements": 2, "content": [
+     "connections": {"numberOfElements": 3, "content": [
          {"connectionId": "vhdxz7abbfirh2lh", "createdAt": 1538482606412, "location": "",
           "platform": "Chrome 69.0.3497.100 on Linux 64-bit",
           "token": "wss://localhost:4443?sessionId=TestSession&token=2ezkertrimk6nttk&role=PUBLISHER&turnUsername=H0EQLL&turnCredential=kjh48u",
@@ -21,16 +21,27 @@ SESSIONS = {"numberOfElements": 2, "content": [
               "mediaOptions": {"hasAudio": True, "audioActive": True, "hasVideo": True, "videoActive": True,
                                "typeOfVideo": "CAMERA", "frameRate": 30,
                                "videoDimensions": "{\"width\":640,\"height\":480}", "filter": {}}}],
-          "subscribers": []}, {"connectionId": "maxawd3ysuj1rxvq", "createdAt": 1538482607659, "location": "",
-                               "platform": "Chrome 69.0.3497.100 on Linux 64-bit",
-                               "token": "wss://localhost:4443?sessionId=TestSession&token=ovj1b4ysuqmcirti&role=PUBLISHER&turnUsername=INOAHN&turnCredential=oujrqd",
-                               "role": "PUBLISHER", "serverData": "", "clientData": "TestClient2", "publishers": [],
-                               "subscribers": [
-                                   {"createdAt": 1538482607799, "streamId": "vhdxz7abbfirh2lh_CAMERA_CLVAU"
-                                    }]}]}, "recording": False},
+          "subscribers": []},
+         {"connectionId": "maxawd3ysuj1rxvq", "createdAt": 1538482607659, "location": "",
+          "platform": "Chrome 69.0.3497.100 on Linux 64-bit",
+          "token": "wss://localhost:4443?sessionId=TestSession&token=ovj1b4ysuqmcirti&role=PUBLISHER&turnUsername=INOAHN&turnCredential=oujrqd",
+          "role": "PUBLISHER", "serverData": "", "clientData": "TestClient2", "publishers": [],
+          "subscribers": [
+              {"createdAt": 1538482607799, "streamId": "vhdxz7abbfirh2lh_CAMERA_CLVAU"
+               }
+          ]},
+         {"connectionId": "maxawc4zsuj1rxva", "createdAt": 1538482607659, "location": "",
+          "platform": "Chrome 69.0.3497.100 on Linux 64-bit",
+          "token": "wss://localhost:4443?sessionId=TestSession&token=ovj1b4ysuqmcirti&role=PUBLISHER&turnUsername=INOAHN&turnCredential=oujrqd",
+          "role": "PUBLISHER", "publishers": [],
+          "subscribers": [
+              {"createdAt": 1538482607799, "streamId": "vhdxz7abbfirh2lh_CAMERA_CLVAU"
+               }
+          ]},
+     ]}, "recording": False},
     {"sessionId": "TestSession2", "createdAt": 1538482606338, "mediaMode": "ROUTED", "recordingMode": "MANUAL",
      "defaultOutputMode": "COMPOSED", "defaultRecordingLayout": "BEST_FIT", "customSessionId": "TestSession",
-     "connections": {"numberOfElements": 2, "content": [
+     "connections": {"numberOfElements": 3, "content": [
          {"connectionId": "vhdxz7abbfirh2lh", "createdAt": 1538482606412, "location": "",
           "platform": "Chrome 69.0.3497.100 on Linux 64-bit",
           "token": "wss://localhost:4443?sessionId=TestSession&token=2ezkertrimk6nttk&role=PUBLISHER&turnUsername=H0EQLL&turnCredential=kjh48u",
@@ -44,7 +55,17 @@ SESSIONS = {"numberOfElements": 2, "content": [
                                "token": "wss://localhost:4443?sessionId=TestSession&token=ovj1b4ysuqmcirti&role=PUBLISHER&turnUsername=INOAHN&turnCredential=oujrqd",
                                "role": "PUBLISHER", "serverData": "", "clientData": "TestClient2", "publishers": [],
                                "subscribers": [
-                                   {"createdAt": 1538482607799, "streamId": "vhdxz7abbfirh2lh_CAMERA_CLVAU"}]}]},
+                                   {"createdAt": 1538482607799, "streamId": "vhdxz7abbfirh2lh_CAMERA_CLVAU"}]},
+         {"connectionId": "ipc_IPCAM_rtsp_A8MJ_91_191_213_49_554_live_mpeg4_sdp", "createdAt": 1582121476379,
+          "location": "unknown", "platform": "IPCAM", "role": "PUBLISHER", "serverData": "MY_IP_CAMERA", "publishers": [
+             {"createdAt": 1582121476439,
+              "streamId": "str_IPC_XC1W_ipc_IPCAM_rtsp_A8MJ_91_191_213_49_554_live_mpeg4_sdp",
+              "rtspUri": "rtsp://91.191.213.49:554/live_mpeg4.sdp",
+              "mediaOptions": {"hasAudio": True, "audioActive": True, "hasVideo": True, "videoActive": True,
+                               "typeOfVideo": "IPCAM", "frameRate": None, "videoDimensions": None, "filter": {},
+                               "adaptativeBitrate": True, "onlyPlayWithSubscribers": True}}], "subscribers": []}
+
+     ]},
      "recording": False}
 ]}
 SECRET = 'MY_SECRET'
@@ -155,13 +176,14 @@ def test_missing_connection(session_instance):
 def test_connections(session_instance):
     conns = list(session_instance.connections)
 
-    assert len(conns) == 2
+    assert len(conns) == 3
     assert conns[0].id == 'vhdxz7abbfirh2lh'
     assert conns[1].id == 'maxawd3ysuj1rxvq'
+    assert conns[2].id == 'maxawc4zsuj1rxva'
 
 
 def test_connections_count(session_instance):
-    assert session_instance.connection_count == 2
+    assert session_instance.connection_count == 3
 
 
 #
@@ -232,6 +254,152 @@ def test_signal_no_connection(session_instance, requests_mock):
         session_instance.signal('MY_TYPE', "Hello world!")
 
 
+#
+# IPCAM
+#
+
+def test_publish_simple(session_instance, requests_mock):
+    new_connection_data = {
+        "connectionId": "ipc_IPCAM_rtsp_A8MJ_91_191_213_49_555_live_mpeg4_sdp",
+        "createdAt": 1582121476380,
+        "location": "unknown",
+        "platform": "IPCAM",
+        "role": "PUBLISHER",
+        "serverData": "",
+        "publishers": [
+            {
+                "createdAt": 1582121476439,
+                "streamId": "str_IPC_XC1W_ipc_IPCAM_rtsp_A8MJ_91_191_213_49_554_live_mpeg4_sdp",
+                "rtspUri": "rtsp://91.191.213.50:554/live_mpeg4.sdp",
+                "mediaOptions": {
+                    "hasAudio": True,
+                    "audioActive": True,
+                    "hasVideo": True,
+                    "videoActive": True,
+                    "typeOfVideo": "IPCAM",
+                    "frameRate": None,
+                    "videoDimensions": None,
+                    "filter": {},
+                    "adaptativeBitrate": True,
+                    "onlyPlayWithSubscribers": True
+                }
+            }
+        ],
+        "subscribers": []
+    }
+
+    a = requests_mock.post(urljoin(URL_BASE, 'api/sessions/TestSession/connection'), json=new_connection_data)
+
+    new_connection = session_instance.publish("rtsp://91.191.213.50:554/live_mpeg4.sdp")
+
+    assert a.last_request.json() == {
+        "type": "IPCAM",
+        "rtspUri": "rtsp://91.191.213.50:554/live_mpeg4.sdp",
+        "adaptativeBitrate": True,
+        "onlyPlayWithSubscribers": True,
+        "data": ''
+    }
+
+    assert a.called
+
+    assert new_connection.id == 'ipc_IPCAM_rtsp_A8MJ_91_191_213_49_555_live_mpeg4_sdp'
+    assert new_connection.role == 'PUBLISHER'
+    assert new_connection.client_data is None
+    assert new_connection.token is None
+    assert new_connection.server_data == ''
+    assert new_connection.created_at == datetime.utcfromtimestamp(1582121476380 / 1000.0)
+    assert new_connection.publishers[0].created_at == datetime.utcfromtimestamp(1582121476439 / 1000.0)
+    assert new_connection.publishers[0].stream_id == 'str_IPC_XC1W_ipc_IPCAM_rtsp_A8MJ_91_191_213_49_554_live_mpeg4_sdp'
+    assert new_connection.publishers[0].rtsp_uri == 'rtsp://91.191.213.50:554/live_mpeg4.sdp'
+    assert new_connection.publishers[0].session_id == 'TestSession'
+    assert new_connection.publishers[0].media_options == new_connection_data['publishers'][0]['mediaOptions']
+
+
+def test_publish_extra(session_instance, requests_mock):
+    new_connection_data = {
+        "connectionId": "ipc_IPCAM_rtsp_A8MJ_91_191_213_49_555_live_mpeg4_sdp",
+        "createdAt": 1582121476380,
+        "location": "unknown",
+        "platform": "IPCAM",
+        "role": "PUBLISHER",
+        "serverData": "TEST_DATA",
+        "publishers": [
+            {
+                "createdAt": 1582121476439,
+                "streamId": "str_IPC_XC1W_ipc_IPCAM_rtsp_A8MJ_91_191_213_49_554_live_mpeg4_sdp",
+                "rtspUri": "rtsp://91.191.213.50:554/live_mpeg4.sdp",
+                "mediaOptions": {
+                    "hasAudio": True,
+                    "audioActive": True,
+                    "hasVideo": True,
+                    "videoActive": True,
+                    "typeOfVideo": "IPCAM",
+                    "frameRate": None,
+                    "videoDimensions": None,
+                    "filter": {},
+                    "adaptativeBitrate": False,
+                    "onlyPlayWithSubscribers": True
+                }
+            }
+        ],
+        "subscribers": []
+    }
+
+    a = requests_mock.post(urljoin(URL_BASE, 'api/sessions/TestSession/connection'), json=new_connection_data)
+
+    new_connection = session_instance.publish("rtsp://91.191.213.50:554/live_mpeg4.sdp", "TEST_DATA",
+                                              adaptive_bitrate=False)
+
+    assert a.last_request.json() == {
+        "type": "IPCAM",
+        "rtspUri": "rtsp://91.191.213.50:554/live_mpeg4.sdp",
+        "adaptativeBitrate": False,
+        "onlyPlayWithSubscribers": True,
+        "data": 'TEST_DATA'
+    }
+
+    assert a.called
+
+    assert new_connection.id == 'ipc_IPCAM_rtsp_A8MJ_91_191_213_49_555_live_mpeg4_sdp'
+    assert new_connection.role == 'PUBLISHER'
+    assert new_connection.client_data is None
+    assert new_connection.token is None
+    assert new_connection.server_data == 'TEST_DATA'
+    assert new_connection.created_at == datetime.utcfromtimestamp(1582121476380 / 1000.0)
+    assert new_connection.publishers[0].created_at == datetime.utcfromtimestamp(1582121476439 / 1000.0)
+    assert new_connection.publishers[0].stream_id == 'str_IPC_XC1W_ipc_IPCAM_rtsp_A8MJ_91_191_213_49_554_live_mpeg4_sdp'
+    assert new_connection.publishers[0].rtsp_uri == 'rtsp://91.191.213.50:554/live_mpeg4.sdp'
+    assert new_connection.publishers[0].session_id == 'TestSession'
+    assert new_connection.publishers[0].media_options == new_connection_data['publishers'][0]['mediaOptions']
+
+
+def test_publish_invalid_session(session_instance, requests_mock):
+    a = requests_mock.post(urljoin(URL_BASE, 'api/sessions/TestSession/connection'), json={}, status_code=404)
+
+    with pytest.raises(OpenViduSessionDoesNotExistsError):
+        session_instance.publish("rtsp://91.191.213.50:554/live_mpeg4.sdp")
+
+    assert a.called
+
+
+def test_publish_value_error(session_instance, requests_mock):
+    a = requests_mock.post(urljoin(URL_BASE, 'api/sessions/TestSession/connection'), json={}, status_code=400)
+
+    with pytest.raises(ValueError):
+        session_instance.publish("rtsp://91.191.213.50:554/live_mpeg4.sdp")
+
+    assert a.called
+
+
+def test_publish_server_error(session_instance, requests_mock):
+    a = requests_mock.post(urljoin(URL_BASE, 'api/sessions/TestSession/connection'), json={}, status_code=500)
+
+    with pytest.raises(OpenViduError):
+        session_instance.publish("rtsp://91.191.213.50:554/live_mpeg4.sdp")
+
+    assert a.called
+
+
 #
 # Fetching
 #
@@ -253,7 +421,7 @@ def test_fetching_session_became_invalid(session_instance, requests_mock):
 
 def test_fetching_changed(session_instance, requests_mock):
     original = deepcopy(SESSIONS['content'][0])
-    original['connections']['numberOfElements'] = 3
+    original['connections']['numberOfElements'] = 4
     original['connections']['content'].append({
         "connectionId": "vhdxz7abbfirh3lh", "createdAt": 1538482606412, "location": "",
         "platform": "Chrome 69.0.3497.100 on Linux 64-bit",
@@ -272,9 +440,9 @@ def test_fetching_changed(session_instance, requests_mock):
 
     is_changed = session_instance.fetch()
 
-    assert session_instance.connection_count == 3
+    assert session_instance.connection_count == 4
 
-    assert list(session_instance.connections)[2].id == 'vhdxz7abbfirh3lh'
+    assert list(session_instance.connections)[3].id == 'vhdxz7abbfirh3lh'
 
     assert is_changed
     assert a.called
@@ -282,7 +450,7 @@ def test_fetching_changed(session_instance, requests_mock):
 
 def test_fetching_changed_fetch_by_parent(openvidu_instance, session_instance, requests_mock):
     original = deepcopy(SESSIONS)
-    original['content'][0]['connections']['numberOfElements'] = 3
+    original['content'][0]['connections']['numberOfElements'] = 4
     original['content'][0]['connections']['content'].append({
         "connectionId": "vhdxz7abbfirh3lh", "createdAt": 1538482606412, "location": "",
         "platform": "Chrome 69.0.3497.100 on Linux 64-bit",
@@ -301,9 +469,9 @@ def test_fetching_changed_fetch_by_parent(openvidu_instance, session_instance, r
 
     is_changed = openvidu_instance.fetch()
 
-    assert session_instance.connection_count == 3
+    assert session_instance.connection_count == 4
 
-    assert list(session_instance.connections)[2].id == 'vhdxz7abbfirh3lh'
+    assert list(session_instance.connections)[3].id == 'vhdxz7abbfirh3lh'
 
     assert is_changed
     assert a.called
